require 'spec_helper'

describe Remind101::Client do
  let(:group_id     ) { 1234 }
  let(:message_id   ) { 4321 }
  let(:subscriber_id) { 789 }
  let(:client       ) { described_class.new }

  describe '#authenticate!' do
    let(:username) { 'eric@remind101.com' }
    let(:password) { 'foobar' }
    let(:auth_token) { 'a21d9240-f446-4a4b-884a-56127fa59617' }

    before do
      stub_request(:post, 'https://api.remind.com/v2/access_tokens')
        .with(body: { user: { email: username, password: password } })
        .to_return(body: "{ \"token\": \"#{auth_token}\" }", headers: { 'Content-Type' => 'application/json' })
    end

    it 'returns an auth token for the user' do
      new_client = client.authenticate!(username, password)
      expect(new_client.auth_token).to eq auth_token
    end
  end

  describe '#get_groups' do
    before { stub_request(:get, 'https://api.remind.com/v2/groups') }

    it 'gets the groups' do
      client.get_groups
    end
  end

  describe '#post_group' do
    before { stub_request(:post, 'https://api.remind.com/v2/groups').with(body: body) }
    let(:body) { { group: { name: 'Math 101' } } }

    it 'creates the group' do
      client.post_group body
    end
  end

  describe '#delete_group' do
    before { stub_request(:delete, "https://api.remind.com/v2/groups/#{group_id}") }

    it 'destroys the group' do
      client.delete_group group_id
    end
  end

  describe '#get_messages' do
    before { stub_request(:get, 'https://api.remind.com/v2/messages') }

    it 'gets the messages' do
      client.get_messages
    end
  end

  describe '#get_message' do
    before { stub_request(:get, 'https://api.remind.com/v2/messages/1') }

    it 'gets the messages' do
      client.get_message(1)
    end
  end

  describe '#post_message' do
    before { stub_request(:post, 'https://api.remind.com/v2/messages').with(body: body) }
    let(:body) { { message: { body: "There's a test tomorrow!", recipients: [{ type: 'group', id: [1] }] } } }

    it 'creates the message' do
      client.post_message body
    end
  end

  describe '#delete_message' do
    before { stub_request(:delete, "https://api.remind.com/v2/messages/#{message_id}") }

    it 'destroys the message' do
      client.delete_message message_id
    end
  end

  describe '#get_chat_message' do
    let(:chat_uuid) { '0bcae64e-12b4-45e3-829a-78a76518c271' }
    let(:message_uuid) { '4b7d7819-1dbd-434a-b9c7-d0adfba61b02' }
    before { stub_request(:get, "https://api.remind.com/v2/chats/#{chat_uuid}/messages/#{message_uuid}") }

    it 'gets the chat message' do
      client.get_chat_message(chat_uuid, message_uuid)
    end
  end

  describe '#get_subscribers' do
    before { stub_request(:get, "https://api.remind.com/v2/subscribers") }

    it 'gets the subscribers' do
      client.get_subscribers
    end
  end

  describe '#get_subscriber' do
    before { stub_request(:get, "https://api.remind.com/v2/subscribers/#{subscriber_id}") }

    it 'gets the subscriber' do
      client.get_subscriber(subscriber_id)
    end
  end

  describe '#rename_subscriber' do
    before { stub_request(:post, "https://api.remind.com/v2/subscribers/#{subscriber_id}/rename").with(body: { name: 'Eric' }) }

    it 'updates the subscriber' do
      client.rename_subscriber(subscriber_id, 'Eric')
    end
  end

  describe '#delete_group_subscriber' do
    before { stub_request(:delete, "https://api.remind.com/v2/groups/#{group_id}/subscribers/#{subscriber_id}") }
    
    it 'removes the subscriber' do
      client.delete_group_subscriber(group_id, subscriber_id)
    end
  end

  describe '#delete_group_subscribers' do
    before { stub_request(:delete, "https://api.remind.com/v2/groups/#{group_id}/subscribers") }
    
    it 'removes the subscriber' do
      client.delete_group_subscribers(group_id)
    end
  end
end
